/***********************************************************************
 *                                                                     *
 * CMDS.H                                                              *
 *   This header contains various commands used in the main message    *
 *   system (including the menu bar, status line, and miscellaneous    *
 *   dialog boxes.)                                                    *
 *                                                                     *
 * HEADER FILE for Multiple Menu Bar Demo                              *
 *                                                                     *
 ***********************************************************************
 *                                                                     *
 * This code was written by Borland Technical Support.                 *
 * It is provided as is with no warranties expressed or implied.       *
 *                                                                     *
 ***********************************************************************/

#ifndef _CMDS_H
#define _CMDS_H

const unsigned cmOne     = 100;
const unsigned cmTwo     = 101;
const unsigned cmThree   = 102;
const unsigned cmCycle   = 110;
const unsigned cmNothing = 111;

#endif
