/*------------------------------------------------------------*/
/* filename -       nminputli.cpp                             */
/*                                                            */
/* defines the streamable name for class TInputLine           */
/*------------------------------------------------------------*/
/*
 *      Turbo Vision - Version 2.0
 *
 *      Copyright (c) 1994 by Borland International
 *      All Rights Reserved.
 *
 */

#if !defined( NO_STREAMABLE )
#define Uses_TInputLine
#include <tvision/tv.h>

const char * const _NEAR TInputLine::name = "TInputLine";
#endif
