/*------------------------------------------------------------*/
/* filename -       nmscoll.cpp                               */
/*                                                            */
/* defines the streamable name for class TSortedCollection    */
/*------------------------------------------------------------*/
/*
 *      Turbo Vision - Version 2.0
 *
 *      Copyright (c) 1994 by Borland International
 *      All Rights Reserved.
 *
 */

#define Uses_TSortedCollection
#include <tvision/tv.h>

const char * const _NEAR TSortedCollection::name = "TSortedCollection";
