/*------------------------------------------------------------*/
/* filename -       nmclustr.cpp                              */
/*                                                            */
/* defines the streamable name for class TCluster             */
/*------------------------------------------------------------*/
/*
 *      Turbo Vision - Version 2.0
 *
 *      Copyright (c) 1994 by Borland International
 *      All Rights Reserved.
 *
 */

#if !defined(NO_STREAMABLE)
#define Uses_TCluster
#include <tvision/tv.h>

const char * const _NEAR TCluster::name = "TCluster";
#endif
